/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.client.gui;

import java.io.File;
import java.util.Enumeration;

import javax.swing.Icon;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import net.sf.distriblend.share.FileMap;
import net.sf.distriblend.share.LocalPath;

/**
 * 
 * @author Ian Thompson
 * 
 */
public class ProjectTreeModel implements TreeModel {

	private final DefaultTreeModel model;

	/**
	 * 
	 */
	public ProjectTreeModel() {
		this.model = new DefaultTreeModel(null);
	}

	/**
	 * @param root
	 *            the root to set
	 */
	public void setRoot(IconedNode root) {
		this.model.setRoot(root);
	}

	/**
	 * Adds the given file or directory to the specified node under the name
	 * provided. If the given file is a directory then the whole directory is
	 * added (without recursing or even adding sub-directories). If an entry
	 * already exists by the same remote name, this method will return with no
	 * effect.
	 * 
	 * @param parent
	 *            The node to which to add the new child node.
	 * @param file
	 *            The file or directory to add, may be <code>null</code> if no
	 *            supporting file is required (ie. for directories or preloaded
	 *            files).
	 * @param remoteName
	 *            The new name of the file. Must be a valid file name.
	 * @param isDirectory
	 *            Whether or not the entry represents a directory.
	 */
	public void addToNode(IconedNode parent, File file, String remoteName,
			boolean isDirectory) {
		Object o = parent.getUserObject();
		FileMap parentMap = o instanceof FileMap ? (FileMap) o : null;

		/* Check nothing exists by the same remoteName (case insensitive). */
		Enumeration<IconedNode> siblings = parent.children();
		for (IconedNode node; siblings.hasMoreElements();) {
			node = siblings.nextElement();
			o = node.getUserObject();
			if (o instanceof FileMap
					&& ((FileMap) o).getDestinationPath().getName()
							.equalsIgnoreCase(remoteName))
				return;
		}

		LocalPath childDestPath;
		if (parentMap == null)
			childDestPath = new LocalPath(remoteName, isDirectory);
		else
			childDestPath = new LocalPath(parentMap.getDestinationPath(),
					remoteName, isDirectory);

		FileMap fileMap = new FileMap(file, childDestPath);
		Icon icon = (file == null ? Icons.fromFileType(remoteName, isDirectory)
				: Icons.fromFileType(file)).getImageIcon();
		IconedNode childNode = new IconedNode(fileMap, icon);
		int last = parent.getChildCount();
		parent.insert(childNode, last);

		/* Check if directory then only add files, so will not recurse again. */
		if (file != null && file.isDirectory()) {
			File[] files = file.listFiles();
			for (File f : files)
				if (!f.isDirectory())
					addToNode(childNode, f, f.getName(), false);
		}

		model.nodesWereInserted(parent, new int[] { last });
	}

	/**
	 * 
	 * 
	 */
	public void clear() {
		IconedNode node = getRoot();
		int count = node.getChildCount();
		for (int i = count - 1; i >= 0; i--)
			node.remove(i);
		model.nodeStructureChanged(getRoot());
	}

	/**
	 * 
	 * 
	 */
	public void reload() {
		model.nodeStructureChanged(getRoot());
	}

	/**
	 * 
	 * @param node
	 */
	public void removeFromParent(IconedNode node) {
		if (!node.equals(getRoot())) {
			IconedNode parent = (IconedNode) node.getParent();
			int i = parent.getIndex(node);
			if (i != -1) {
				node.removeFromParent();
				model.nodesWereRemoved(parent, new int[] { i },
						new Object[] { node });
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#addTreeModelListener(javax.swing.event.TreeModelListener)
	 */
	public void addTreeModelListener(TreeModelListener l) {
		model.addTreeModelListener(l);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#getChild(java.lang.Object, int)
	 */
	public Object getChild(Object parent, int index) {
		return model.getChild(parent, index);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#getChildCount(java.lang.Object)
	 */
	public int getChildCount(Object parent) {
		return model.getChildCount(parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#getIndexOfChild(java.lang.Object,
	 *      java.lang.Object)
	 */
	public int getIndexOfChild(Object parent, Object child) {
		return model.getIndexOfChild(parent, child);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#getRoot()
	 */
	public IconedNode getRoot() {
		return (IconedNode) model.getRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#isLeaf(java.lang.Object)
	 */
	public boolean isLeaf(Object node) {
		return model.isLeaf(node);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#removeTreeModelListener(javax.swing.event.TreeModelListener)
	 */
	public void removeTreeModelListener(TreeModelListener l) {
		model.removeTreeModelListener(l);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.tree.TreeModel#valueForPathChanged(javax.swing.tree.TreePath,
	 *      java.lang.Object)
	 */
	public void valueForPathChanged(TreePath path, Object newValue) {
		model.nodeChanged((IconedNode) path.getLastPathComponent());
	}
}
