/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor;

import java.util.Queue;
import java.util.Vector;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Logger;

import net.sf.distriblend.distributor.storage.Project;
import net.sf.distriblend.distributor.storage.Storage;
import net.sf.distriblend.distributor.storage.StorageException;
import net.sf.distriblend.distributor.storage.User;
import net.sf.distriblend.share.Task;
import net.sf.distriblend.share.TaskFrame;

/**
 * A globally shared queue for distributing tasks.
 * 
 * @author Ian Thompson
 * 
 */
public class TaskQueue {

	/**
	 * Returns the single instance of this queue.
	 * 
	 * @return The single instance of this queue.
	 */
	public static TaskQueue instance() {
		if (instance == null)
			instance = new TaskQueue();
		return instance;
	}

	/**
	 * Checks that the specified task is ready for distribution before adding it
	 * to the queue.
	 * 
	 * @param task
	 *            The task to add.
	 * @return Whether the task was ready to be added for distribution.
	 */
	public synchronized boolean add(Task task) {
		if (!readyForDistribution(task))
			return false;
		return queue.add(task);
	}

	/**
	 * Removes all entries from the queue originating from the specified user
	 * and project. If any entry is found to be incomplete, the method will
	 * return <code>false</code> and have no effect.
	 * 
	 * @param user
	 *            The name of the user.
	 * @param projectName
	 *            The name of the project.
	 * @return Whether all tasks are complete and thus removed.
	 */
	public synchronized boolean checkAndDelete(String user, String projectName) {
		Vector<Task> toRemove = new Vector<Task>();
		for (Task t : queue) {
			if (t.getUser().equals(user)
					&& t.getProjectName().equals(projectName)) {
				if (!t.isComplete())
					return false;
				toRemove.add(t);
			}
		}
		queue.removeAll(toRemove);
		return true;
	}

	/**
	 * Attempts to obtain the next available task frame. If none are available
	 * <code>null</code> is returned.
	 * 
	 * @return The next available task frame, or <code>null</code> if the
	 *         queue is empty.
	 */
	public synchronized TaskFrame next() {
		Task task = queue.peek();

		/* Remove any complete tasks from the front of the queue. */
		while (task != null && task.isComplete()) {
			queue.remove();
			task = queue.peek();
		}
		if (task == null)
			return null;
		int frame = task.checkOutFrame();
		return new TaskFrame(task, frame);
	}

	/**
	 * Checks the frame back into the task and informs the parent
	 * {@link Project} of the change. If the task is thus completed it is
	 * removed from the queue. This is the preferred method for checking frames
	 * in for a task due to its synchronization with the rest of the queue
	 * system.
	 * 
	 * @param taskFrame
	 *            The frame to check in.
	 * @param success
	 *            The success of the processing of the frame.
	 */
	public synchronized void finish(TaskFrame taskFrame, boolean success) {
		Task task = taskFrame.getTask();
		task.checkIn(taskFrame.getFrame(), success);
		if (task.isComplete()) {
			queue.remove(task);
		}
		User user = Storage.instance().getUser(task.getUser());
		try {
			user.getProject(task.getProjectName()).saveTask(task);
		} catch (StorageException e) {
			logger.severe("Failed to update task file");
		}
	}

	/**
	 * Checks the frame back into the task, putting it back on the queue.
	 * 
	 * @param taskFrame
	 *            The frame to check in.
	 */
	public synchronized void cancel(TaskFrame taskFrame) {
		Task task = taskFrame.getTask();
		task.cancel(taskFrame.getFrame());
	}

	/** The single instance of this queue. */
	private static TaskQueue instance;

	/** The logger for this class. */
	private static Logger logger = Logger.getLogger(TaskQueue.class.getName());

	/** The task queue. */
	private final Queue<Task> queue;

	/**
	 * Creates a new task queue.
	 */
	private TaskQueue() {
		this.queue = new LinkedBlockingQueue<Task>();
	}

	/**
	 * Returns <code>true</code> only if the user and project names have been
	 * set on the specified task.
	 * 
	 * @param e
	 *            The task to test.
	 * @return Whether the task is ready for distribution.
	 */
	private boolean readyForDistribution(Task e) {
		if (e.getUser() == null)
			return false;
		if (e.getProjectName() == null)
			return false;
		return true;
	}
}
