/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor.gui;

import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import net.sf.distriblend.share.Settings;

/**
 * Presents a wizard for configuring the distributor.
 * 
 * @author Ian Thompson
 * 
 */
public class Wizard extends JDialog {

	/**
	 * Used to indicate that this wizard was dismissed by closing or clicking
	 * Cancel.
	 */
	public static final int CANCEL_OPTION = 0;

	/** Used to indicate that this wizard has been completed. */
	public static final int FINISH_OPTION = 1;

	/**
	 * @return selected option used to dismiss the dialog.
	 */
	public static int showConfigurationDialog() {
		Wizard gui = new Wizard();
		gui.setModal(true);
		gui.pack();
		gui.setVisible(true);
		if (gui.returnOption == FINISH_OPTION)
			logger.info("Wizard complete.");
		else
			logger.warning("Wizard was closed. Distributor will not start.");
		return gui.returnOption;
	}

	/**
	 * This method initializes previousButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getPreviousButton() {
		if (previousButton == null) {
			previousButton = new JButton();
			previousButton.setText("Prev");
			previousButton.setMnemonic(KeyEvent.VK_P);
			previousButton.setEnabled(false);
			previousButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							CardLayout cl = (CardLayout) getWizardPanel()
									.getLayout();

							if (stepIndex > 0) {
								stepIndex--;
								cl.show(getWizardPanel(), "wizardStep"
										+ stepIndex);
								if (stepIndex == 0)
									getPreviousButton().setEnabled(false);
								if (stepIndex == getWizardSteps().length - 2) {
									getNextButton().setText("Next");
									getNextButton().setMnemonic(KeyEvent.VK_N);
								}
							}
						}
					});
		}
		return previousButton;
	}

	/**
	 * This method initializes nextButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getNextButton() {
		if (nextButton == null) {
			nextButton = new JButton();
			nextButton.setText("Next");
			nextButton.setMnemonic(KeyEvent.VK_N);
			nextButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent evt) {
					CardLayout cl = (CardLayout) getWizardPanel().getLayout();

					if (stepIndex < getWizardSteps().length - 1) {
						if (!getWizardSteps()[stepIndex].validateForm())
							return;

						stepIndex++;
						cl.show(getWizardPanel(), "wizardStep" + stepIndex);
						getWizardSteps()[stepIndex].stepEntered();
						getPreviousButton().setEnabled(true);
						if (stepIndex == getWizardSteps().length - 1) {
							getNextButton().setText("Finish");
							getNextButton().setMnemonic(KeyEvent.VK_F);
						}
					} else if (getWizardSteps()[stepIndex].validateForm()) {
						try {
							WizardStep4 last = (WizardStep4) getWizardSteps()[3];
							Settings.instance().save(last.getFile());
							logger.info("Settings saved.");
							JOptionPane.showMessageDialog(Wizard.this,
									"Settings saved.", "Save",
									JOptionPane.INFORMATION_MESSAGE);
						} catch (IOException e) {
							JOptionPane.showMessageDialog(Wizard.this,
									"Save failed", "Save Failed",
									JOptionPane.ERROR_MESSAGE);
							e.printStackTrace();
						}
						returnOption = FINISH_OPTION;
						dispose();
					}
				}
			});
		}
		return nextButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.setMnemonic(KeyEvent.VK_C);
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes wizardPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWizardPanel() {
		if (wizardPanel == null) {
			wizardPanel = new JPanel();
			wizardPanel.setLayout(new CardLayout());
			for (int i = 0; i < getWizardSteps().length; i++)
				wizardPanel.add(getWizardSteps()[i], "wizardStep" + i);
		}
		return wizardPanel;
	}

	/**
	 * This method initializes wizardStep1
	 * 
	 * @return net.sf.distriblend.distributor.gui.WizardStep1
	 */
	private WizardStep[] getWizardSteps() {
		if (wizardSteps == null) {
			wizardSteps = new WizardStep[] { new WizardStep1(),
					new WizardStep2(), new WizardStep3(), new WizardStep4() };
		}
		return wizardSteps;
	}

	/**
	 * This method initializes
	 * 
	 */
	public Wizard() {
		super();
		initialize();
	}

	/* -- Private -- */

	private JPanel mainPanel = null;
	private JButton previousButton = null;
	private JButton nextButton = null;
	private JButton cancelButton = null;
	private JPanel wizardPanel = null;
	private WizardStep[] wizardSteps = null;
	private int stepIndex = 0;

	private int returnOption = CANCEL_OPTION;
	private static final Logger logger = Logger.getLogger(Wizard.class
			.getName());

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.setTitle("Distributor Setup Wizard");
		Image image = Toolkit.getDefaultToolkit().getImage(
				getClass().getResource("trayicon.png"));
		this.setIconImage(image);
		this.setPreferredSize(new Dimension(500, 350));
		this.setContentPane(getMainPanel());
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	}

	/**
	 * This method initializes mainPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridwidth = 3;
			gridBagConstraints3.insets = new Insets(4, 4, 4, 4);
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.weighty = 1.0;
			gridBagConstraints3.gridy = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 2;
			gridBagConstraints2.insets = new Insets(16, 16, 4, 4);
			gridBagConstraints2.anchor = GridBagConstraints.SOUTHEAST;
			gridBagConstraints2.weighty = 0.0;
			gridBagConstraints2.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.insets = new Insets(16, 4, 4, 4);
			gridBagConstraints1.weighty = 0.0;
			gridBagConstraints1.anchor = GridBagConstraints.SOUTHWEST;
			gridBagConstraints1.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.insets = new Insets(16, 16, 4, 4);
			gridBagConstraints.anchor = GridBagConstraints.SOUTHEAST;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 0.0;
			gridBagConstraints.gridy = 1;
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.add(getPreviousButton(), gridBagConstraints);
			mainPanel.add(getNextButton(), gridBagConstraints1);
			mainPanel.add(getCancelButton(), gridBagConstraints2);
			mainPanel.add(getWizardPanel(), gridBagConstraints3);
		}
		return mainPanel;
	}
}
