/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor.storage;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.TreeSet;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sf.distriblend.distributor.TaskQueue;
import net.sf.distriblend.share.FileMap;
import net.sf.distriblend.share.LocalPath;
import net.sf.distriblend.share.Task;
import net.sf.distriblend.share.protocol.FileTransfer.Header;

/**
 * 
 * @author Ian Thompson
 * 
 */
public class Project {

	/* -- Encapsulation methods. -- */

	/**
	 * Returns the project's name.
	 * 
	 * @return The project's name.
	 */
	public String getProjectName() {
		return projectName;
	}

	/* -- Project file methods -- */

	public Archive makeArchiveForUpload() {
		synchronized (archiveList) {
			Archive a = new Archive(archiveList.size());
			archiveList.add(a);
			return a;
		}
	}

	/**
	 * Returns the number of archives stored for this project.
	 * 
	 * @return The number of archives stored for this project.
	 */
	public int getArchiveCount() {
		return archiveList.size();
	}

	/**
	 * 
	 * @param index
	 * @return Archive file
	 */
	public Archive getArchive(int index) {
		return archiveList.get(index);
	}

	/**
	 * @return List of all files
	 * @throws IOException
	 */
	public LocalPath[] listFiles() throws IOException {

		TreeSet<LocalPath> paths = new TreeSet<LocalPath>();
		synchronized (ARCHIVE_FILES_LOCK) {
			File[] files = archivesDir.listFiles();
			ObjectInputStream in;
			for (File indexFile : files) {
				if (!indexFile.getName().endsWith(".index"))
					continue;
				in = new ObjectInputStream(new BufferedInputStream(
						new FileInputStream(indexFile)));
				try {
					while (true) {
						Header header = (Header) in.readObject();
						paths.add(header.getPath());
					}
				} catch (EOFException e) {
					// OK
				} catch (IOException e) {
					throw e;
				} catch (ClassNotFoundException e) {
					throw new IOException(e);
				}
			}
		}
		return paths.toArray(new LocalPath[paths.size()]);
	}

	/* -- Project task methods -- */

	/**
	 * 
	 * @param task
	 * @throws StorageException
	 * @return Task index to identify the task for further storage operations.
	 */
	public int addTask(Task task) throws StorageException {

		if (task == null)
			throw new NullPointerException("Task cannot be null.");

		int taskID;
		synchronized (TASK_FILES_LOCK) {
			int i = tasks.size();
			File taskFile;
			do {
				taskFile = new File(tasksDir, "task_" + numberFormat.format(i)
						+ ".task");
				i++;
			} while (taskFile.exists());
			taskID = i - 1;

			ObjectOutputStream out = null;
			try {
				out = new ObjectOutputStream(new BufferedOutputStream(
						new FileOutputStream(taskFile)));
				out.writeUnshared(task);
			} catch (IOException e) {
				throw new StorageException("Failed to write task to disk.", e);
			} finally {
				try {
					if (out != null)
						out.close();
				} catch (IOException e) {
					throw new StorageException("Failed to write task to disk.",
							e);
				}
			}

			tasks.add(new StoredTask(task, taskID));
		}
		task.setUser(user.getUserName());
		task.setProjectName(projectName);
		TaskQueue.instance().add(task);
		return taskID;
	}

	/**
	 * Updates the disk file version of the specified task.
	 * 
	 * @param task
	 * @throws StorageException
	 */
	public void saveTask(Task task) throws StorageException {

		synchronized (tasks) {
			StoredTask matchingTask = null;
			for (StoredTask storedTask : tasks) {
				if (storedTask.task == task) {
					matchingTask = storedTask;
					break;
				}
			}
			if (matchingTask == null)
				throw new StorageException("Task not found.");

			synchronized (TASK_FILES_LOCK) {
				File taskFile = new File(tasksDir, "task_"
						+ numberFormat.format(matchingTask.taskID) + ".task");
				try {
					ObjectOutputStream out = new ObjectOutputStream(
							new BufferedOutputStream(new FileOutputStream(
									taskFile)));
					out.writeUnshared(task);
					out.close();
				} catch (IOException e) {
					logger.log(Level.SEVERE, "Failed to save task to disk.", e);
				}
			}
		}
	}

	/**
	 * 
	 * @return incomplete tasks
	 */
	public int countTotalFrames() {
		int count = 0;
		for (StoredTask t : tasks)
			count += t.task.getFrameCount();
		return count;
	}

	/**
	 * 
	 * @return complete tasks
	 */
	public int countRenderedFrames() {
		int count = 0;
		for (StoredTask t : tasks)
			count += t.task.getCompletedFrameCount();
		return count;
	}

	/**
	 * Retrieves a file suitable for storing the frame produced by the specified
	 * sub task. Note: this method only returns a path, it makes no attempt to
	 * create directories and gives no indication that it is possible to create
	 * such a directory or even write to it.
	 * 
	 * @param task
	 * @param frame
	 * @return User storage directory.
	 */
	public File frameFile(Task task, int frame) {
		String sourceFile = task.getSourceFile().getName();
		String fileName = numberFormat.format(frame) + "."
				+ task.getImageFormat().extensions[0];

		return new File(framesDir, sourceFile + File.separator + fileName);
	}

	/**
	 * (not synchronized with storage of frames, only use when no new frames
	 * will be added)
	 * 
	 * @return file maps for archiving
	 */
	public FileMap[] getFrameFileMaps() {
		Vector<FileMap> fileMaps = new Vector<FileMap>();
		File[] sourceFileDirs = framesDir.listFiles();
		File[] frameFiles;
		LocalPath parent;
		for (File sourceFileDir : sourceFileDirs) {
			frameFiles = sourceFileDir.listFiles();
			parent = new LocalPath(sourceFileDir.getName(), true);
			for (File frameFile : frameFiles) {
				fileMaps.add(new FileMap(frameFile, new LocalPath(parent,
						frameFile.getName(), false)));
			}
		}

		return fileMaps.toArray(new FileMap[fileMaps.size()]);
	}

	/* -- Private -- */

	/** The logger for this class. */
	private static final Logger logger = Logger.getLogger(Project.class
			.getName());

	/** Prefix used on archives while uploading. */
	private static final String uploadPrefix = "uploading_";

	/** Prefix used on archives. */
	private static final String archivePrefix = "archive_";

	/** The user to which this project belongs. */
	private final User user;

	/** A formatter for forcing four-digit (or greater) numbers. */
	private final NumberFormat numberFormat;

	/** The name of this project. */
	private final String projectName;

	/** ROOT/USER/STORE/PROJECTS/PROJECT */
	private final File projectStorage;

	/** ROOT/USER/STORE/PROJECTS/PROJECT/archives */
	private final File archivesDir;

	/** ROOT/USER/STORE/PROJECTS/PROJECT/tasks */
	private final File tasksDir;

	/** ROOT/USER/STORE/PROJECTS/PROJECT/frames */
	private final File framesDir;

	/** Synchronization lock for task files. */
	private final Object TASK_FILES_LOCK = new Object();

	/** Synchronization lock for archive files. */
	private final Object ARCHIVE_FILES_LOCK = new Object();

	/** A maintained list of tasks for this project. */
	private final Vector<StoredTask> tasks;

	/** A maintained list of file archives for this project. */
	private final List<Archive> archiveList;

	/**
	 * Reads tasks from disk. Files must be of the form
	 * <code>"task_NUM.task"</code> where NUM is an integer. Also reads in and
	 * holds a list of uploaded archive files.
	 * 
	 * @param user
	 * @param projectName
	 * @throws IOException
	 * @throws StorageException
	 */
	Project(User user, String projectName) throws IOException, StorageException {

		this.user = user;

		/* Set up 4-digit (or more) number formatting for file names. */
		this.numberFormat = NumberFormat.getNumberInstance();
		this.numberFormat.setMinimumIntegerDigits(4);
		this.numberFormat.setGroupingUsed(false);

		this.projectName = projectName;
		this.projectStorage = new File(user.getProjectsDirectory(), projectName);
		projectStorage.mkdir();

		/* Make archive and frame files directories. */
		this.archivesDir = new File(projectStorage, "archives");
		this.archivesDir.mkdir();
		this.framesDir = new File(projectStorage, "frames");
		this.framesDir.mkdir();

		/* Make tasks directory and read. */
		this.tasksDir = new File(projectStorage, "tasks");
		this.tasksDir.mkdir();
		this.tasks = new Vector<StoredTask>();
		this.archiveList = Collections
				.synchronizedList(new ArrayList<Archive>());

		/* Make archive files list prior to loading tasks. */
		String[] archiveNames = archivesDir.list();
		int i = 0;
		for (String name : archiveNames)
			if (name.startsWith(archivePrefix) && name.endsWith(".data"))
				archiveList.add(new Archive(i++));

		/* Load task and submit for processing. */
		for (File f : tasksDir.listFiles()) {
			String name = f.getName();
			int prefix = name.indexOf("task_");
			int suffix = name.indexOf(".task");
			int taskID = Integer.parseInt(name.substring(prefix + 5, suffix));
			if (prefix == 0 && suffix == name.length() - 5 && taskID >= 0) {
				ObjectInputStream in = new ObjectInputStream(
						new BufferedInputStream(new FileInputStream(f)));
				try {
					Object o = in.readObject();
					if (o instanceof Task) {
						Task task = (Task) o;
						logger.log(Level.FINE,
								"Loading task {0} for user {1}.", new Object[] {
										task, user.getUserName() });
						this.tasks.add(new StoredTask(task, taskID));

						task.setUser(user.getUserName());
						task.setProjectName(projectName);
						TaskQueue.instance().add(task);
					} else
						throw new StorageException("Unexpected type: "
								+ o.getClass().getName());
				} catch (ClassNotFoundException e) {
					throw new StorageException(e.getMessage());
				} finally {
					in.close();
				}
			}
		}
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	private class StoredTask {
		Task task;

		int taskID;

		/**
		 * 
		 * @param task
		 * @param index
		 */
		public StoredTask(Task task, int index) {
			super();
			this.task = task;
			this.taskID = index;
		}
	}

	/**
	 * 
	 * @author Ian Thompson
	 * 
	 */
	public class Archive {

		/**
		 * @return the dataFile
		 */
		public File getDataFile() {
			return dataFile;
		}

		/**
		 * @return the indexFile
		 */
		public File getIndexFile() {
			return indexFile;
		}

		/**
		 * @return the tempDataFile
		 */
		public File getTempDataFile() {
			return tempDataFile;
		}

		/**
		 * @return finalized
		 */
		public boolean isFinalized() {
			return finalized;
		}

		/**
		 * 
		 * @return the result of the rename operation.
		 */
		public boolean finalizeUpload() {
			boolean b;
			synchronized (ARCHIVE_FILES_LOCK) {
				b = tempDataFile.renameTo(dataFile);
			}
			// When traversing the list this field is checked hence volatility
			finalized = true;
			return b;
		}

		/**
		 * @return Success of deletion.
		 */
		public boolean delete() {
			return tempDataFile.delete() && dataFile.delete()
					&& indexFile.delete();
		}

		/**
		 * Creates a new archive. This archive must be added to the archive
		 */
		Archive(int index) {
			String baseName = archivePrefix + numberFormat.format(index);
			dataFile = new File(archivesDir, baseName + ".data");
			indexFile = new File(archivesDir, baseName + ".index");
			tempDataFile = new File(archivesDir, uploadPrefix + baseName
					+ ".data");
		}

		/* -- Private -- */

		private final File tempDataFile, dataFile, indexFile;
		private volatile boolean finalized;
	}
}
