/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.distributor.storage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sf.distriblend.distributor.TaskQueue;
import net.sf.distriblend.share.HashFunction;

/**
 * 
 * @author Ian Thompson
 * 
 */
public class User {

	/* -- Encapsulation methods. -- */

	/**
	 * Returns the user's name.
	 * 
	 * @return The user's name.
	 */
	public String getUserName() {
		return userName;
	}

	/**
	 * Returns the {@link Project} found with name equal to the specified name
	 * or <code>null</code> if none is found.
	 * 
	 * @param projectName
	 *            The name of the project to find.
	 * @return The project with the given name or <code>null</code> if no
	 *         project is found matching this name.
	 */
	public Project getProject(String projectName) {
		return projects.get(projectName);
	}

	/* -- Project handling methods -- */

	/**
	 * 
	 * @return A list of project names owned by the specified user or
	 *         <code>null</code> if the user's profile does not exist.
	 */
	public String[] listProjects() {
		synchronized (projects) {
			return projects.keySet().toArray(new String[projects.size()]);
		}
	}

	/**
	 * 
	 * @param projectName
	 * @return Whether or not the specified project name is in the user's list.
	 */
	public boolean hasProject(String projectName) {
		return projects.containsKey(projectName);
	}

	/**
	 * @param projectName
	 * @throws StorageException
	 * @throws IOException
	 */
	public void makeProject(String projectName) throws StorageException,
			IOException {
		synchronized (projects) {
			if (!projects.containsKey(projectName))
				projects.put(projectName, new Project(this, projectName));
		}
	}

	/**
	 * @param projectName
	 * @throws StorageException
	 */
	public void deleteProject(String projectName) throws StorageException {
		Project project;
		synchronized (projects) {
			project = projects.get(projectName);
			if (project == null)
				throw new StorageException("Project does not exist.");

			if (!TaskQueue.instance().checkAndDelete(userName, projectName))
				throw new StorageException("Project is still being processed.");

			projects.remove(projectName);
		}

		/* Delete files. */
		File file = new File(projectsDir, projectName);
		purge(file);
	}

	private void purge(File file) {
		File[] files = file.listFiles();
		if (files != null)
			for (File f : files)
				purge(f);
		file.delete();
	}

	/* -- Package private -- */

	/**
	 * 
	 * @param name
	 * @param userStorage
	 * @throws IOException
	 * @throws StorageException
	 */
	User(String name, File userStorage) throws IOException, StorageException {

		this.userName = name;
		this.userStorage = userStorage;

		/* Extract password. */
		this.password = null;
		File passwordFile = new File(this.userStorage, "user.pwd");
		if (userStorage.isDirectory() && passwordFile.exists()) {
			BufferedReader r = new BufferedReader(new FileReader(passwordFile));
			this.password = r.readLine();
			r.close();
		} else {
			throw new StorageException(
					"User storage must contain user.pwd file.");
		}

		/* List projects. */
		this.projects = Collections
				.synchronizedMap(new TreeMap<String, Project>());
		this.projectsDir = new File(userStorage, "projects");

		/* Check existence and attempt to create projects directory. */
		if (!projectsDir.exists() && !projectsDir.mkdir())
			throw new StorageException(
					"User storage must contain the writable directory 'projects'.");

		File[] files = projectsDir.listFiles();
		for (File file : files) {
			if (file.isDirectory()) {
				String projectName = file.getName();
				logger.log(Level.FINE, "Adding project {0} for user {1}.",
						new Object[] { projectName, name });
				Project p = new Project(this, projectName);
				projects.put(projectName, p);
			}
		}
	}

	/**
	 * 
	 * @param hashedPassword
	 *            The password to check hashed once with the standard default
	 *            hash and again with the default hash and salt value.
	 * @param salt
	 *            The salt to use for the second hashing iteration.
	 * @return Whether or not the password matches the one stored in the user's
	 *         profile.
	 * @throws NoSuchAlgorithmException
	 */
	boolean checkHashedPassword(String hashedPassword, String salt)
			throws NoSuchAlgorithmException {
		String hashed = HashFunction.getDefault().hash(password + salt);
		if (hashed.equals(hashedPassword))
			return true;
		else
			return false;
	}

	/**
	 * 
	 * @return
	 */
	File getProjectsDirectory() {
		return projectsDir;
	}

	/* -- Private -- */

	/** The logger for this class. */
	private static Logger logger = Logger.getLogger(User.class.getName());

	/** The user's name. */
	private final String userName;

	/** The user's password. */
	private String password;

	/** {ROOT}/{USER} */
	private final File userStorage;

	/** {ROOT}/{USER}/projects */
	private final File projectsDir;

	/** A map from names to projects. */
	private final Map<String, Project> projects;
}