/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.node;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.NumberFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sf.distriblend.share.Settings;
import net.sf.distriblend.share.Task;
import net.sf.distriblend.share.TaskFrame;

/**
 * Manages the node application's storage directories. These consist of the
 * following paths:
 * <p>
 * <table border=1>
 * <tr>
 * <td><code>{ROOT}/{USER}/{PROJECT}/index</code>
 * <td>Archive index files
 * <tr>
 * <td><code>{ROOT}/{USER}/{PROJECT}/files</code>
 * <td>All project files
 * <tr>
 * <td><code>{ROOT}/{USER}/{PROJECT}/frames</code>
 * <td>Rendered frames </table>
 * </p>
 * for each user and project.
 * 
 * @author Ian Thompson
 * 
 */
public class Directories {

	/**
	 * Returns the single instance of the {@link Directories} class.
	 * 
	 * @return Directories instance.
	 */
	public static Directories instance() {
		if (instance == null) {
			instance = new Directories();
		}
		return instance;
	}

	/**
	 * Attempts to delete all files and sub-directories in the root storage
	 * directory.
	 */
	public void purge() {
		File[] files = root.listFiles();
		if (files != null)
			for (File f : files)
				purge(f);
	}

	private void purge(File file) {
		File[] files = file.listFiles();
		if (files != null)
			for (File f : files)
				purge(f);
		file.delete();
	}

	/**
	 * Returns the directory where archive index files should be stored.
	 * 
	 * @param user
	 *            The name of the user who owns the project.
	 * @param projectName
	 *            The name of the project.
	 * @return The directory for archive index files.
	 */
	public File archiveStorage(String user, String projectName) {

		/* {ROOT}/{USER}/{PROJECT}/index */
		return new File(root, user + File.separator + projectName
				+ File.separator + "index");
	}

	/**
	 * Returns an abstract path name that may be used to locate or store the
	 * index of an archive uploaded to the node.
	 * 
	 * @param user
	 * @param projectName
	 * @param archiveID
	 * @return The location for storage and retrieval of the index of an archive
	 *         uploaded to the node.
	 */
	public File indexFile(String user, String projectName, int archiveID) {

		/* {ROOT}/{USER}/{PROJECT}/index/archive_####.index */
		return new File(archiveStorage(user, projectName), "archive_"
				+ numbers.format(archiveID) + ".index");
	}

	/**
	 * Returns the number of archives received for a particular project.
	 * 
	 * @param user
	 *            The name of the user that owns the project.
	 * @param projectName
	 *            The name of the project.
	 * @return The number of archives received.
	 */
	public int countArchives(String user, String projectName) {
		String[] list = archiveStorage(user, projectName).list();
		if (list == null)
			return 0;
		return list.length;
	}

	/**
	 * Returns the directory where all files related to the specified project
	 * should be stored.
	 * 
	 * @param user
	 *            The name of the user who owns the project.
	 * @param projectName
	 *            The name of the project.
	 * @return The directory for all project files.
	 */
	public File filesStorage(String user, String projectName) {

		/* {ROOT}/{USER}/{PROJECT} */
		return new File(root, user + File.separator + projectName
				+ File.separator + "files");
	}

	/**
	 * Returns the base directory where all rendered frames should be stored.
	 * <p>
	 * The frames themselves are stored in sub-directories reflecting the source
	 * file used to produce them. For example if the file <blockquote>
	 * <code>{ROOT}/{USER}/{PROJECT}/files/sub/dir/scene.blend</code>
	 * </blockquote> was used to source frame 5, then the output frame should be
	 * stored in <blockquote>
	 * <code>{ROOT}/{USER}/{PROJECT}/frames/sub/dir/scene.blend/0005.png</code>
	 * </blockquote> This method returns the
	 * <code>{ROOT}/{USER}/{PROJECT}/frames</code> portion.
	 * 
	 * @param user
	 *            The name of the user who owns the project.
	 * @param projectName
	 *            The name of the project.
	 * @return The directory for all project files.
	 */
	public File framesStorage(String user, String projectName) {

		/* {ROOT}/{USER}/{PROJECT} */
		return new File(root, user + File.separator + projectName
				+ File.separator + "frames");
	}

	/**
	 * Returns an abstract path name that may be used to locate or store the
	 * image produced by the specified task frame.
	 * 
	 * @param taskFrame
	 *            A task frame producing a frame file.
	 * @return The location for storage and retrieval of the image generated by
	 *         the given task frame.
	 */
	public File frameFile(TaskFrame taskFrame) {
		Task task = taskFrame.getTask();
		String user = task.getUser();
		String projectName = task.getProjectName();
		String sourceFile = task.getSourceFile().getPath();
		String frame = numbers.format(taskFrame.getFrame());
		String extension = task.getImageFormat().extensions[0];
		String sep = File.separator;

		/* {ROOT}/{USER}/{PROJECT}/frames/{SRCFILE}/####.??? */
		return new File(framesStorage(user, projectName), sourceFile + sep
				+ frame + "." + extension);
	}

	/**
	 * Tests whether the given archive name is listed for the project and user
	 * specified. If an archive <em>is</em> listed its files should be present
	 * on the node system.
	 * 
	 * @param user
	 *            The name of a user.
	 * @param projectName
	 *            The name of a project owned by the specified user.
	 * @param archiveName
	 *            The name of the archive to check for.
	 * @return Whether the archive is listed.
	 */
	public synchronized boolean archiveListed(String user, String projectName,
			String archiveName) {
		File list = new File(filesStorage(user, projectName), "archives");

		/* If list exists check if listed. */
		if (list.exists()) {
			try {
				BufferedReader in = new BufferedReader(new FileReader(list));
				String entry;
				while ((entry = in.readLine()) != null) {
					if (entry.equals(archiveName))
						return true;
				}
				in.close();
			} catch (IOException e) {
				logger.log(Level.SEVERE, "Failed to read archive list.");
				logger.log(Level.SEVERE, e.getLocalizedMessage());
				return false;
			}
		}
		return false;
	}

	/**
	 * Adds the specified archive name to the list maintained by this class.
	 * This method should be called if and only if the archive has been uploaded
	 * and extracted successfully.
	 * 
	 * @param user
	 *            The name of a user.
	 * @param projectName
	 *            The name of a project owned by the given user.
	 * @param archiveName
	 *            The name to add to the list.
	 */
	public synchronized void addArchiveName(String user, String projectName,
			String archiveName) {

		/* Already listed? Do nothing. */
		if (archiveListed(user, projectName, archiveName))
			return;

		/* Not already listed, append to file. */
		File list = new File(filesStorage(user, projectName), "archives");
		try {
			PrintWriter out = new PrintWriter(new BufferedWriter(
					new FileWriter(list, true)));
			out.println(archiveName);
			out.close();
		} catch (IOException e) {
			logger.log(Level.SEVERE, "Failed to write archive list.");
			logger.log(Level.SEVERE, e.getLocalizedMessage());
		}
	}

	/* -- Private -- */

	/** Single Directories instance. */
	private static Directories instance = null;

	/** The {@link Logger} for this class. */
	private static Logger logger = Logger
			.getLogger(Directories.class.getName());

	/** A number formatter for producing four (or greater) digit numbers. */
	private static NumberFormat numbers = NumberFormat.getNumberInstance();
	static {
		numbers.setMinimumIntegerDigits(4);
		numbers.setGroupingUsed(false);
	}

	/** The root directory for all files held by the node. */
	private final File root;

	/**
	 * Private constructor to maintain a single instance.
	 */
	private Directories() {
		this.root = new File(Settings.instance().getStringProperty(
				"storage.root"));
	}
}
