/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007-2008: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share;

/**
 * An open alternative to the proprietary encoder. Based on trial and error with
 * existing keys and the understanding that bits are in network order.
 * 
 * The process works by taking 6bit chunks out of an array of 8bit bytes,
 * calculating the integral value and mapping it onto a 64 character set:
 * 'A'-'Z', 'a'-'z', '0'-'9', '+', '/'
 * 
 * @author Ian Thompson
 * 
 */
public class Base64Encoder {

	/**
	 * Encodes the byte array in 64bit format.
	 * 
	 * @param raw
	 *            the array to encode.
	 * @return the encoded text.
	 */
	public static String encode(byte[] raw) {

		StringBuilder sb = new StringBuilder();

		int numbits = raw.length * 8;
		int spare = numbits % 6;

		for (int bit = 0; bit < numbits; bit += 6) {

			int curbyte = bit / 8;
			int curval = (raw[curbyte] & 0xFF);
			if (curval < 0)
				curval += 256;
			int bitofs = bit % 8;
			int nxtval, val = 0;
			switch (bitofs) {
			case 0:
				val = (curval >> 2) & 0x3F;
				break;
			case 2:
				val = (curval >> 0) & 0x3F;
				break;
			case 4:
				nxtval = curbyte < raw.length - 1 ? raw[curbyte + 1] : 0;
				if (nxtval < 0)
					nxtval += 256;
				val = ((curval << 2) + (nxtval >> 6)) & 0x3F;
				break;
			case 6:
				nxtval = curbyte < raw.length - 1 ? raw[curbyte + 1] : 0;
				if (nxtval < 0)
					nxtval += 256;
				val = ((curval << 4) + (nxtval >> 4)) & 0x3F;
				break;
			}
			sb.append(chars[val]);
		}
		if (spare == 2) // Final byte has two bits remaining
			sb.append("==");
		else if (spare == 4)
			sb.append('=');

		return sb.toString();
	}

	/**
	 * Character map.
	 */
	private static char[] chars;
	static {
		chars = new char[64];
		for (int i = 0; i < 26; i++) {
			chars[i] = (char) ('A' + i);
			chars[i + 26] = (char) ('a' + i);
		}
		for (int i = 0; i < 10; i++) {
			chars[i + 52] = (char) ('0' + i);
		}
		chars[62] = '+';
		chars[63] = '/';
	}
}
