/*
 * Distriblend: A distributed rendering system for Blender
 * Copyright (C) 2007: Ian Philip Thompson
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share;

import java.io.Serializable;
import java.util.StringTokenizer;

/**
 * Represents a relative path to a local resource. The path is stored in a
 * platform independent format where the character <code>/</code> is used to
 * separate path components.
 * 
 * @author Ian Thompson
 * 
 */
public class LocalPath implements Serializable, Comparable<LocalPath> {

	/**
	 * The separator character used by this class.
	 */
	public static final String separator = "/";

	/**
	 * Creates a path from the given path name and indicates whether it should
	 * be treated as a directory. If the path is invalid an exception will be
	 * thrown.
	 * <p>
	 * A valid path name must satisfy the following:
	 * <ul>
	 * <li>start with a non-separator character
	 * <li>contain no two adjacent separators <code>//</code>
	 * <li>contain only {@linkplain SafeString#isSafe(String) safe Strings}
	 * between separator characters.
	 * 
	 * @param path
	 *            The path name to use as the path.
	 * @param isDirectory
	 *            Whether the specified path should be treated as a directory
	 *            (or a normal file).
	 * @throws IllegalArgumentException
	 *             If the given path name is not a valid path.
	 */
	public LocalPath(String path, boolean isDirectory)
			throws IllegalArgumentException {
		this.path = path;
		this.directory = isDirectory;
		validate();
	}

	/**
	 * Creates a path by appending the given path name to the specified parent
	 * path name. If the resulting path is invalid an exception will be thrown.
	 * <p>
	 * A valid path name must satisfy the following:
	 * <ul>
	 * <li>start with a non-separator character
	 * <li>contain no two adjacent separators <code>//</code>
	 * <li>contain only {@linkplain SafeString#isSafe(String) safe Strings}
	 * between separator characters.
	 * 
	 * @param parent
	 * @param path
	 * @param isDirectory
	 * @throws IllegalArgumentException
	 */
	public LocalPath(LocalPath parent, String path, boolean isDirectory)
			throws IllegalArgumentException {
		if (!parent.isDirectory())
			throw new IllegalArgumentException(
					"Files may only be added to directories.");
		this.path = parent.getPath() + separator + path;
		this.directory = isDirectory;
		validate();
	}

	/**
	 * Returns a String representation of this path.
	 * 
	 * @return This path in a String format.
	 */
	public String getPath() {
		return path;
	}

	/**
	 * Returns the name of the last path component. If this path points to a
	 * file, this method will return the file name. For directories the
	 * directory name will be returned.
	 * 
	 * @return The name of the last path component.
	 */
	public String getName() {
		int index = path.lastIndexOf(separator);
		return path.substring(index + 1);
	}

	/**
	 * Returns a {@link LocalPath} representing the parent directory or
	 * <code>null</code> if this path has no visible parent.
	 * 
	 * @return This parent path of this path or <code>null</code> if there is
	 *         no parent component.
	 */
	public LocalPath getParent() {
		int index = path.lastIndexOf(separator);
		if (index == -1)
			return null;
		return new LocalPath(path.substring(0, index), true);
	}

	/**
	 * Returns <code>true</code> if this path represents a directory.
	 * 
	 * @return Whether this path represents a directory.
	 */
	public boolean isDirectory() {
		return directory;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getPath();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(LocalPath o) {
		return getPath().compareTo(o.getPath());
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof LocalPath) {
			LocalPath localPath = (LocalPath) obj;
			return getPath().equals(localPath.getPath());
		} else
			return false;
	}

	/**
	 * The normalized {@link String} representation of this path.
	 */
	private String path;

	/**
	 * Directory flag to indicate whether this path represents a directory.
	 */
	private boolean directory;

	/**
	 * Validates the path. If invalid, an {@link IllegalArgumentException} is
	 * thrown.
	 * 
	 * @throws IllegalArgumentException
	 *             If the path contains invalid components.
	 */
	private void validate() throws IllegalArgumentException {
		if (path.startsWith(separator)
				|| path.indexOf(separator + separator) != -1)
			throw new IllegalArgumentException("Path invalid: " + path);

		StringTokenizer t = new StringTokenizer(path, separator);
		while (t.hasMoreTokens()) {
			String s = t.nextToken();
			if (s.length() == 0 || !SafeString.isSafe(s)) {
				throw new IllegalArgumentException("Path invalid: " + path);
			}
		}

		/* Normalize by removing any trailing separator. */
		if (path.endsWith(separator))
			path = path.substring(0, path.length() - 1);
	}
}
