/*
 * Distriblend: A distributed rendering system for Blender
 * 
 * Copyright (C) 2007: Ian Philip Thompson
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sf.distriblend.share.protocol;

import java.io.IOException;
import java.io.InputStream;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.io.Serializable;

import net.sf.distriblend.share.LocalPath;
import net.sf.distriblend.share.net.event.TransferEvent;
import net.sf.distriblend.share.net.event.TransferListener;

/**
 * This class contains messages for use in other protocols for file transfer.
 * Transfer takes place in the following manner:
 * <ul>
 * <li>Header: Contains information about the file such as its name, size and
 * where to put it.
 * <li>Raw data: The file contents of the same length as provided in the
 * Header.
 * </ul>
 * 
 * @author Ian Thompson
 * 
 */
public final class FileTransfer {

	/**
	 * Transfer header message. Holds information about the file about to be
	 * transferred.
	 * 
	 * @author Ian Thompson
	 * 
	 */
	public static final class Header implements Serializable {

		/**
		 * Forms a header for the transfer of a single file.
		 * 
		 * @param path
		 *            The destination path for the file.
		 * @param size
		 *            The size of the file.
		 */
		public Header(LocalPath path, long size) {
			this.path = path;
			this.size = size;
			if (!isValid())
				throw new IllegalArgumentException("Size must be positive.");
		}

		/**
		 * Returns the name of the file.
		 * 
		 * @return The name of the file.
		 */
		public String getName() {
			return path.getName();
		}

		/**
		 * Returns the path of the file.
		 * 
		 * @return The path of the file.
		 */
		public LocalPath getPath() {
			return path;
		}

		/**
		 * Returns the size of the file.
		 * 
		 * @return The size of the file.
		 */
		public long getSize() {
			return size;
		}

		/* -- Private -- */

		/**
		 * The destination path of the file.
		 */
		private final LocalPath path;

		/**
		 * The size of the file.
		 */
		private final long size;

		/**
		 * Reads and validates an instance of this class from the stream
		 * provided.
		 * 
		 * @param s
		 *            The stream from which to read the object.
		 * @throws IOException
		 *             If an error occurs while reading from the stream.
		 * @throws ClassNotFoundException
		 */
		private void readObject(ObjectInputStream s) throws IOException,
				ClassNotFoundException {
			s.defaultReadObject();

			/* Check for tampering since constructor validation. */
			if (!isValid())
				throw new NotSerializableException(this.getClass().getName());
		}

		/**
		 * Checks that the fields for this request are valid and suitable. If
		 * so, <code>true</code> is returned. Otherwise, <code>false</code>
		 * is returned.
		 * 
		 * @return Whether all fields are valid and suitable for this request.
		 */
		private boolean isValid() {
			if (path == null)
				return false;
			if (size < 0)
				return false;
			return true;
		}
	}

	/* -- Static methods -- */

	/**
	 * Transfers the specified amount of data from the input stream to the
	 * output stream.
	 * 
	 * @param in
	 *            The stream from which to read.
	 * @param out
	 *            The stream to which data will be written.
	 * @param size
	 *            The amount of data to transfer.
	 * @return The actual amount of data transferred.
	 * @throws IOException
	 *             if an IO error occurs.
	 */
	public static long transferData(InputStream in, OutputStream out, long size)
			throws IOException {
		return transferData(in, out, size, null);
	}

	/**
	 * Transfers the specified amount of data from the input stream to the
	 * output stream.
	 * 
	 * @param in
	 *            The stream from which to read.
	 * @param out
	 *            The stream to which data will be written.
	 * @param size
	 *            The amount of data to transfer.
	 * @param listener
	 *            The listener to receive updates on the progress of the
	 *            transfer.
	 * @return The actual amount of data transferred.
	 * @throws IOException
	 *             if an IO error occurs.
	 */
	public static long transferData(InputStream in, OutputStream out,
			long size, TransferListener listener) throws IOException {

		int bufferSize = size < 1024 ? (int) size : 1024;
		byte[] buffer = new byte[bufferSize];
		long transferred = 0;
		if (listener != null)
			listener.progressChanged(new TransferEvent(0));
		int len;
		while ((len = in.read(buffer, 0, bufferSize)) > 0) {
			out.write(buffer, 0, len);
			transferred += len;
			if (listener != null)
				listener.progressChanged(new TransferEvent(transferred));
			if (transferred >= size)
				break;
			bufferSize = size - transferred < 1024 ? (int) (size - transferred)
					: 1024;
		}
		out.flush();
		return transferred;
	}

	/* -- Private -- */

	/** Private constructor to prevent instantiation. */
	private FileTransfer() {
	}
}
